#ifndef CONF_H_
#define CONF_H_

typedef enum {
	CONF_TYPE_UNKNOWN,
	CONF_TYPE_INTEGER,
	CONF_TYPE_REAL,
	CONF_TYPE_STRING,
	CONF_TYPE_BOOL,
} config_val_type_t;

union config_value {
	long int	integer;
	char		*str;
	float		real;
	boolean_t	bool;
};

struct cmd {
	const char		*section;
	const char		*key;
	union config_value	value;
};

typedef enum {
	CONF_ERR_UNKNOWN = -1,
	CONF_ERR_SECT_NAME,
	CONF_ERR_KEY_NAME,
	CONF_ERR_KEY_NAME_FORMAT,
	CONF_ERR_KEY_OUT_SECT,
	CONF_ERR_KEY_VALUE,
	CONF_ERR_EQU_NOT_FOUND,
	CONF_ERR_EXTR_INT,
	CONF_ERR_EXTR_REAL,
	CONF_ERR_EXTR_STR,
	CONF_ERR_EXTR_BOOL,
	CONF_ERR_UNEXPECT_TOKEN,
	CONF_ERR_UNDEF_SECTION
} config_error_type_t;

struct config_file;

typedef int	(*key_handler_t)(struct cmd *, void *);
typedef void	(*section_handler_t)(void *);
typedef void	(*error_handler_t)(struct config_file *, config_error_type_t,
				   const char *);

struct key_option {
	const char		*name;
	config_val_type_t	val_type;
	key_handler_t		handler;
};

struct section_option {
	const char		*name;
	struct key_option	*keys;
	section_handler_t	handler;
};

struct config_file {
	FILE			*fp;
	char			*file_name;
	unsigned int		line_number;
	struct section_option	*current;

	struct section_option	*sections;
	error_handler_t		error_handler;
	void			*opaque;
};

struct config_file *config_create(const char *, struct section_option *, void *);
int config_parse(struct config_file *);
void config_destroy(struct config_file **);


#endif /* CONF_H_ */
